@@ -1,49 +1,45 @@
 """Arch Linux installer - guided, templates etc."""
+import curses
 import importlib
 import os
 import sys
 import time
-import curses
 import traceback
 from argparse import ArgumentParser, Namespace
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Union
+from typing import TYPE_CHECKING, Any
 
-from .lib import disk
-from .lib import menu
-from .lib import models
-from .lib import packages
-from .lib import exceptions
-from .lib import luks
-from .lib import locale
-from .lib import mirrors
-from .lib import networking
-from .lib import profile
-from .lib import interactions
 from . import default_profiles
-
-from .lib.hardware import SysInfo, GfxDriver
-from .lib.installer import Installer, accessibility_tools_in_use
-from .lib.output import FormattedOutput, log, error, debug, warn, info
-from .lib.pacman import Pacman
-from .lib.storage import storage
-from .lib.global_menu import GlobalMenu
+from .lib import disk, exceptions, interactions, locale, luks, mirrors, models, networking, packages, profile
 from .lib.boot import Boot
-from .lib.translationhandler import TranslationHandler, Language, DeferredTranslation
-from .lib.plugins import plugins, load_plugin
 from .lib.configuration import ConfigurationOutput
-
 from .lib.general import (
-	generate_password, locate_binary, clear_vt100_escape_codes,
-	JSON, UNSAFE_JSON, SysCommandWorker, SysCommand,
-	run_custom_user_commands, json_stream_to_structure, secret
+	JSON,
+	UNSAFE_JSON,
+	SysCommand,
+	SysCommandWorker,
+	clear_vt100_escape_codes,
+	generate_password,
+	json_stream_to_structure,
+	locate_binary,
+	run_custom_user_commands,
+	secret,
 )
+from .lib.global_menu import GlobalMenu
+from .lib.hardware import GfxDriver, SysInfo
+from .lib.installer import Installer, accessibility_tools_in_use
+from .lib.output import FormattedOutput, debug, error, info, log, warn
+from .lib.pacman import Pacman
+from .lib.plugins import load_plugin, plugins
+from .lib.storage import storage
+from .lib.translationhandler import DeferredTranslation, Language, TranslationHandler
+from .tui import Tui
 
 if TYPE_CHECKING:
 	_: Any
 
 
-__version__ = "2.8.6"
+__version__ = "3.0.1"
 storage['__version__'] = __version__
 
 # add the custom _ as a builtin, it can now be used anywhere in the
@@ -58,7 +54,7 @@ debug(f"Virtualization detected: {SysInfo.virtualization()}; is VM: {SysInfo.is_
 debug(f"Graphics devices detected: {SysInfo._graphics_devices().keys()}")
 
 # For support reasons, we'll log the disk layout pre installation to match against post-installation layout
-debug(f"Disk states before installing: {disk.disk_layouts()}")
+debug(f"Disk states before installing:\n{disk.disk_layouts()}")
 
 parser = ArgumentParser()
 
@@ -102,7 +98,7 @@ if 'sphinx' not in sys.modules:
 		exit(1)
 
 
-def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err: bool = False) -> dict:
+def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err: bool = False) -> dict:  # type: ignore[type-arg]
 	"""We accept arguments not defined to the parser. (arguments "ad hoc").
 	Internally argparse return to us a list of words so we have to parse its contents, manually.
 	We accept following individual syntax for each argument
@@ -112,10 +108,11 @@ def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err:
 		--argument   (boolean as default)
 	the optional parameters to the function alter a bit its behaviour:
 	* multiple allows multivalued arguments, each value separated by whitespace. They're returned as a list
-	* error. If set any non correctly specified argument-value pair to raise an exception. Else, simply notifies the existence of a problem and continues processing.
+	* error. If set any non correctly specified argument-value pair to raise an exception. Else, simply notifies the
+	existence of a problem and continues processing.
 
-	To a certain extent, multiple and error are incompatible. In fact, the only error this routine can catch, as of now, is the event
-	argument value value ...
+	To a certain extent, multiple and error are incompatible. In fact, the only error this routine can catch, as of now,
+	is the event argument value value ...
 	which isn't am error if multiple is specified
 	"""
 	tmp_list = [arg for arg in unknowns if arg != "="]  # wastes a few bytes, but avoids any collateral effect of the destructive nature of the pop method()
@@ -150,7 +147,7 @@ def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err:
 	return config
 
 
-def cleanup_empty_args(args: Union[Namespace, Dict]) -> Dict:
+def cleanup_empty_args(args: Namespace | dict) -> dict:  # type: ignore[type-arg]
 	"""
 	Takes arguments (dictionary or argparse Namespace) and removes any
 	None values. This ensures clean mergers during dict.update(args)
@@ -169,21 +166,24 @@ def cleanup_empty_args(args: Union[Namespace, Dict]) -> Dict:
 	return clean_args
 
 
-def get_arguments() -> Dict[str, Any]:
+def get_arguments() -> dict[str, Any]:
 	""" The handling of parameters from the command line
 	Is done on following steps:
 	0) we create a dict to store the arguments and their values
 	1) preprocess.
-		We take those arguments which use JSON files, and read them into the argument dict. So each first level entry becomes a argument on it's own right
+		We take those arguments which use JSON files, and read them into the argument dict. So each first level entry
+		becomes a argument on it's own right
 	2) Load.
-		We convert the predefined argument list directly into the dict via the vars() function. Non specified arguments are loaded with value None or false if they are booleans (action="store_true").
-		The name is chosen according to argparse conventions. See above (the first text is used as argument name, but underscore substitutes dash)
-		We then load all the undefined arguments. In this case the names are taken as written.
+		We convert the predefined argument list directly into the dict via the vars() function. Non specified arguments
+		are loaded with value None or false if they are booleans (action="store_true"). The name is chosen according to
+		argparse conventions. See above (the first text is used as argument name, but underscore substitutes dash). We
+		then load all the undefined arguments. In this case the names are taken as written.
 		Important. This way explicit command line arguments take precedence over configuration files.
 	3) Amend
-		Change whatever is needed on the configuration dictionary (it could be done in post_process_arguments but  this ougth to be left to changes anywhere else in the code, not in the arguments dictionary
+		Change whatever is needed on the configuration dictionary (it could be done in post_process_arguments but  this
+		ougth to be left to changes anywhere else in the code, not in the arguments dictionary
 	"""
-	config: Dict[str, Any] = {}
+	config: dict[str, Any] = {}
 	args, unknowns = parser.parse_known_args()
 	# preprocess the JSON files.
 	# TODO Expand the url access to the other JSON file arguments ?
@@ -255,11 +255,10 @@ def load_config() -> None:
 		arguments['audio_config'] = models.AudioConfiguration.parse_arg(arguments['audio_config'])
 
 	if arguments.get('disk_encryption', None) is not None and disk_config is not None:
-		password = arguments.get('encryption_password', '')
 		arguments['disk_encryption'] = disk.DiskEncryption.parse_arg(
 			arguments['disk_config'],
 			arguments['disk_encryption'],
-			password
+			arguments.get('encryption_password', '')
 		)
 
 
@@ -279,13 +278,13 @@ def post_process_arguments(arguments: dict[str, Any]) -> None:
 
 
 define_arguments()
-arguments: Dict[str, Any] = get_arguments()
+arguments: dict[str, Any] = get_arguments()
 post_process_arguments(arguments)
 
 
 # @archinstall.plugin decorator hook to programmatically add
 # plugins in runtime. Useful in profiles_bck and other things.
-def plugin(f, *args, **kwargs) -> None:
+def plugin(f, *args, **kwargs) -> None:  # type: ignore[no-untyped-def]
 	plugins[f.__name__] = f
 
 
@@ -331,24 +330,6 @@ def main() -> None:
 	importlib.import_module(mod_name)
 
 
-def _shutdown_curses() -> None:
-	try:
-		curses.nocbreak()
-
-		try:
-			from archinstall.tui.curses_menu import tui
-			tui.screen.keypad(False)
-		except Exception:
-			pass
-
-		curses.echo()
-		curses.curs_set(True)
-		curses.endwin()
-	except Exception:
-		# this may happen when curses has not been initialized
-		pass
-
-
 def run_as_a_module() -> None:
 	exc = None
 
@@ -358,7 +339,7 @@ def run_as_a_module() -> None:
 		exc = e
 	finally:
 		# restore the terminal to the original state
-		_shutdown_curses()
+		Tui.shutdown()
 
 		if exc:
 			err = ''.join(traceback.format_exception(exc))
