@@ -1,23 +1,19 @@
-from typing import Optional, List, Any, TYPE_CHECKING
-
-from archinstall.default_profiles.profile import ProfileType, GreeterType
+from archinstall.default_profiles.profile import GreeterType, ProfileType
 from archinstall.default_profiles.xorg import XorgProfile
 
-if TYPE_CHECKING:
-	_: Any
-
 
 class I3wmProfile(XorgProfile):
 	def __init__(self) -> None:
 		super().__init__('i3-wm', ProfileType.WindowMgr, description='')
 
 	@property
-	def packages(self) -> List[str]:
+	def packages(self) -> list[str]:
 		return [
 			'i3-wm',
 			'i3lock',
 			'i3status',
 			'i3blocks',
+			'xss-lock',
 			'xterm',
 			'lightdm-gtk-greeter',
 			'lightdm',
@@ -25,5 +21,5 @@ class I3wmProfile(XorgProfile):
 		]
 
 	@property
-	def default_greeter_type(self) -> Optional[GreeterType]:
+	def default_greeter_type(self) -> GreeterType | None:
 		return GreeterType.Lightdm
