@@ -1,23 +1,18 @@
-from typing import Optional, List, Any, TYPE_CHECKING
-
-from archinstall.default_profiles.profile import ProfileType, GreeterType
+from archinstall.default_profiles.profile import GreeterType, ProfileType
 from archinstall.default_profiles.xorg import XorgProfile
 
-if TYPE_CHECKING:
-	_: Any
-
 
 class QtileProfile(XorgProfile):
 	def __init__(self) -> None:
 		super().__init__('Qtile', ProfileType.WindowMgr, description='')
 
 	@property
-	def packages(self) -> List[str]:
+	def packages(self) -> list[str]:
 		return [
 			'qtile',
 			'alacritty'
 		]
 
 	@property
-	def default_greeter_type(self) -> Optional[GreeterType]:
+	def default_greeter_type(self) -> GreeterType | None:
 		return GreeterType.Lightdm
