@@ -1,7 +1,6 @@
-from typing import List, Union, TYPE_CHECKING
+from typing import TYPE_CHECKING
 
 import archinstall
-
 from archinstall.default_profiles.profile import Profile, ProfileType
 from archinstall.lib.models import User
 
@@ -17,15 +16,15 @@ class DockerProfile(Profile):
 		)
 
 	@property
-	def packages(self) -> List[str]:
+	def packages(self) -> list[str]:
 		return ['docker']
 
 	@property
-	def services(self) -> List[str]:
+	def services(self) -> list[str]:
 		return ['docker']
 
 	def post_install(self, install_session: 'Installer') -> None:
-		users: Union[User, List[User]] = archinstall.arguments.get('!users', [])
+		users: User | list[User] = archinstall.arguments.get('!users', [])
 		if not isinstance(users, list):
 			users = [users]
 
