@@ -1,4 +1,4 @@
-from typing import List, TYPE_CHECKING
+from typing import TYPE_CHECKING
 
 from archinstall.default_profiles.profile import Profile, ProfileType
 
@@ -15,11 +15,11 @@ class PostgresqlProfile(Profile):
 		)
 
 	@property
-	def packages(self) -> List[str]:
+	def packages(self) -> list[str]:
 		return ['postgresql']
 
 	@property
-	def services(self) -> List[str]:
+	def services(self) -> list[str]:
 		return ['postgresql']
 
 	def post_install(self, install_session: 'Installer') -> None:
