@@ -1,4 +1,4 @@
-from typing import Any, Optional, TYPE_CHECKING, List
+from typing import TYPE_CHECKING, Any
 
 from archinstall.default_profiles.profile import Profile, ProfileType
 
@@ -22,7 +22,7 @@ class XorgProfile(Profile):
 			advanced=advanced
 		)
 
-	def preview_text(self) -> Optional[str]:
+	def preview_text(self) -> str | None:
 		text = str(_('Environment type: {}')).format(self.profile_type.value)
 		if packages := self.packages_text():
 			text += f'\n{packages}'
@@ -30,7 +30,7 @@ class XorgProfile(Profile):
 		return text
 
 	@property
-	def packages(self) -> List[str]:
+	def packages(self) -> list[str]:
 		return [
 			'xorg-server'
 		]
