@@ -1,79 +1,36 @@
 from pathlib import Path
-from typing import TYPE_CHECKING, Callable, Optional
 
 import archinstall
-from archinstall import Installer
-from archinstall import profile
-from archinstall import SysInfo
-from archinstall import disk
-from archinstall import menu
-from archinstall import models
-from archinstall import locale
-from archinstall import info, debug
+from archinstall import SysInfo, debug, info
+from archinstall.lib import disk, locale
+from archinstall.lib.configuration import ConfigurationOutput
+from archinstall.lib.global_menu import GlobalMenu
+from archinstall.lib.installer import Installer
+from archinstall.lib.interactions.general_conf import ask_chroot
+from archinstall.lib.models import AudioConfiguration, Bootloader
+from archinstall.lib.models.network_configuration import NetworkConfiguration
+from archinstall.lib.profile.profiles_handler import profile_handler
+from archinstall.tui import Tui
 
-if TYPE_CHECKING:
-	_: Callable[[str], str]
+if archinstall.arguments.get('help'):
+	print("See `man archinstall` for help.")
+	exit(0)
 
 
 def ask_user_questions() -> None:
-	global_menu = archinstall.GlobalMenu(data_store=archinstall.arguments)
+	"""
+	First, we'll ask the user for a bunch of user input.
+	Not until we're satisfied with what we want to install
+	will we continue with the actual installation steps.
+	"""
 
-	global_menu.enable('archinstall-language')
+	with Tui():
+		global_menu = GlobalMenu(data_store=archinstall.arguments)
 
-	# Set which region to download packages from during the installation
-	global_menu.enable('mirror_config')
+		if not archinstall.arguments.get('advanced', False):
+			global_menu.set_enabled('parallel downloads', False)
 
-	global_menu.enable('locale_config')
-
-	global_menu.enable('disk_config', mandatory=True)
-
-	# Specify disk encryption options
-	global_menu.enable('disk_encryption')
-
-	# Ask which boot-loader to use (will only ask if we're in UEFI mode, otherwise will default to GRUB)
-	global_menu.enable('bootloader')
-
-	global_menu.enable('swap')
-
-	# Get the hostname for the machine
-	global_menu.enable('hostname')
-
-	# Ask for a root password (optional, but triggers requirement for super-user if skipped)
-	global_menu.enable('!root-password', mandatory=True)
-
-	global_menu.enable('!users', mandatory=True)
-
-	# Ask for archinstall-specific profiles_bck (such as desktop environments etc)
-	global_menu.enable('profile_config')
-
-	# Ask about audio server selection if one is not already set
-	global_menu.enable('audio_config')
-
-	# Ask for preferred kernel:
-	global_menu.enable('kernels', mandatory=True)
-
-	global_menu.enable('packages')
-
-	if archinstall.arguments.get('advanced', False):
-		# Enable parallel downloads
-		global_menu.enable('parallel downloads')
-
-	# Ask or Call the helper function that asks the user to optionally configure a network.
-	global_menu.enable('network_config')
-
-	global_menu.enable('timezone')
-
-	global_menu.enable('ntp')
-
-	global_menu.enable('additional-repositories')
-
-	global_menu.enable('__separator__')
-
-	global_menu.enable('save_config')
-	global_menu.enable('install')
-	global_menu.enable('abort')
-
-	global_menu.run()
+		global_menu.run()
 
 
 def perform_installation(mountpoint: Path) -> None:
@@ -88,6 +45,7 @@ def perform_installation(mountpoint: Path) -> None:
 	# Retrieve list of additional repositories and set boolean values appropriately
 	enable_testing = 'testing' in archinstall.arguments.get('additional-repositories', [])
 	enable_multilib = 'multilib' in archinstall.arguments.get('additional-repositories', [])
+	run_mkinitcpio = not archinstall.arguments.get('uki')
 	locale_config: locale.LocaleConfiguration = archinstall.arguments['locale_config']
 	disk_encryption: disk.DiskEncryption = archinstall.arguments.get('disk_encryption', None)
 
@@ -109,11 +67,12 @@ def perform_installation(mountpoint: Path) -> None:
 				installation.generate_key_files()
 
 		if mirror_config := archinstall.arguments.get('mirror_config', None):
-			installation.set_mirrors(mirror_config)
+			installation.set_mirrors(mirror_config, on_target=False)
 
 		installation.minimal_installation(
 			testing=enable_testing,
 			multilib=enable_multilib,
+			mkinitcpio=run_mkinitcpio,
 			hostname=archinstall.arguments.get('hostname', 'archlinux'),
 			locale_config=locale_config
 		)
@@ -124,14 +83,17 @@ def perform_installation(mountpoint: Path) -> None:
 		if archinstall.arguments.get('swap'):
 			installation.setup_swap('zram')
 
-		if archinstall.arguments.get("bootloader") == models.Bootloader.Grub and SysInfo.has_uefi():
+		if archinstall.arguments.get("bootloader") == Bootloader.Grub and SysInfo.has_uefi():
 			installation.add_additional_packages("grub")
 
-		installation.add_bootloader(archinstall.arguments["bootloader"])
+		installation.add_bootloader(
+			archinstall.arguments["bootloader"],
+			archinstall.arguments.get('uki', False)
+		)
 
 		# If user selected to copy the current ISO network configuration
 		# Perform a copy of the config
-		network_config = archinstall.arguments.get('network_config', None)
+		network_config: NetworkConfiguration | None = archinstall.arguments.get('network_config', None)
 
 		if network_config:
 			network_config.install_network_config(
@@ -142,17 +104,17 @@ def perform_installation(mountpoint: Path) -> None:
 		if users := archinstall.arguments.get('!users', None):
 			installation.create_users(users)
 
-		audio_config: Optional[models.AudioConfiguration] = archinstall.arguments.get('audio_config', None)
+		audio_config: AudioConfiguration | None = archinstall.arguments.get('audio_config', None)
 		if audio_config:
 			audio_config.install_audio_config(installation)
 		else:
 			info("No audio server will be installed")
 
 		if archinstall.arguments.get('packages', None) and archinstall.arguments.get('packages', None)[0] != '':
-			installation.add_additional_packages(archinstall.arguments.get('packages', []))
+			installation.add_additional_packages(archinstall.arguments.get('packages', None))
 
 		if profile_config := archinstall.arguments.get('profile_config', None):
-			profile.profile_handler.install_profile_config(installation, profile_config)
+			profile_handler.install_profile_config(installation, profile_config)
 
 		if timezone := archinstall.arguments.get('timezone', None):
 			installation.set_timezone(timezone)
@@ -183,24 +145,42 @@ def perform_installation(mountpoint: Path) -> None:
 		info("For post-installation tips, see https://wiki.archlinux.org/index.php/Installation_guide#Post-installation")
 
 		if not archinstall.arguments.get('silent'):
-			prompt = str(_('Would you like to chroot into the newly created installation and perform post-installation configuration?'))
-			choice = menu.Menu(prompt, menu.Menu.yes_no(), default_option=menu.Menu.yes()).run()
-			if choice.value == menu.Menu.yes():
+			with Tui():
+				chroot = ask_chroot()
+
+			if chroot:
 				try:
 					installation.drop_to_shell()
-				except Exception:
+				except:
 					pass
 
-	debug(f"Disk states after installing: {disk.disk_layouts()}")
+	debug(f"Disk states after installing:\n{disk.disk_layouts()}")
 
 
-ask_user_questions()
+def _guided() -> None:
+	if not archinstall.arguments.get('silent'):
+		ask_user_questions()
 
-fs_handler = disk.FilesystemHandler(
-	archinstall.arguments['disk_config'],
-	archinstall.arguments.get('disk_encryption', None)
-)
+	config = ConfigurationOutput(archinstall.arguments)
+	config.write_debug()
+	config.save()
 
-fs_handler.perform_filesystem_operations()
+	if archinstall.arguments.get('dry_run'):
+		exit(0)
 
-perform_installation(archinstall.storage.get('MOUNT_POINT', Path('/mnt')))
+	if not archinstall.arguments.get('silent'):
+		with Tui():
+			if not config.confirm_config():
+				debug('Installation aborted')
+				_guided()
+
+	fs_handler = disk.FilesystemHandler(
+		archinstall.arguments['disk_config'],
+		archinstall.arguments.get('disk_encryption', None)
+	)
+
+	fs_handler.perform_filesystem_operations()
+	perform_installation(archinstall.storage.get('MOUNT_POINT', Path('/mnt')))
+
+
+_guided()
