@@ -1,63 +1,71 @@
 from __future__ import annotations
 
 import json
-import os
 import logging
+import os
 import time
-import uuid
+from collections.abc import Iterable
 from pathlib import Path
-from typing import List, Dict, Any, Optional, TYPE_CHECKING, Literal, Iterable
+from typing import Any, Literal
 
-from parted import (
-	Disk, Geometry, FileSystem,
-	PartitionException, DiskException,
-	getDevice, getAllDevices, newDisk, freshDisk, Partition, Device
-)
-
-from .device_model import (
-	DeviceModification, PartitionModification,
-	BDevice, _DeviceInfo, _PartitionInfo,
-	FilesystemType, Unit, PartitionTable,
-	ModificationStatus, get_lsblk_info, find_lsblk_info, LsblkInfo,
-	_BtrfsSubvolumeInfo, get_all_lsblk_info, DiskEncryption, LvmVolumeGroup, LvmVolume, Size, LvmGroupInfo,
-	SectorSize, LvmVolumeInfo, LvmPVInfo, SubvolumeModification, BtrfsMountOption
-)
+from parted import Device, Disk, DiskException, FileSystem, Geometry, IOException, Partition, PartitionException, freshDisk, getAllDevices, getDevice, newDisk
 
 from ..exceptions import DiskError, UnknownFilesystemFormat
-from ..general import SysCommand, SysCallError, JSON, SysCommandWorker
+from ..general import SysCallError, SysCommand, SysCommandWorker
 from ..luks import Luks2
-from ..output import debug, error, info, warn, log
+from ..output import debug, error, info, log, warn
 from ..utils.util import is_subpath
-
-if TYPE_CHECKING:
-	_: Any
+from .device_model import (
+	BDevice,
+	BtrfsMountOption,
+	DeviceModification,
+	DiskEncryption,
+	FilesystemType,
+	LsblkInfo,
+	LvmGroupInfo,
+	LvmPVInfo,
+	LvmVolume,
+	LvmVolumeGroup,
+	LvmVolumeInfo,
+	ModificationStatus,
+	PartitionFlag,
+	PartitionGUID,
+	PartitionModification,
+	PartitionTable,
+	SectorSize,
+	Size,
+	SubvolumeModification,
+	Unit,
+	_BtrfsSubvolumeInfo,
+	_DeviceInfo,
+	_PartitionInfo,
+	find_lsblk_info,
+	get_all_lsblk_info,
+	get_lsblk_info,
+	get_lsblk_output,
+)
 
 
-class DeviceHandler(object):
+class DeviceHandler:
 	_TMP_BTRFS_MOUNT = Path('/mnt/arch_btrfs')
 
 	def __init__(self) -> None:
-		self._devices: Dict[Path, BDevice] = {}
+		self._devices: dict[Path, BDevice] = {}
 		self.load_devices()
 
 	@property
-	def devices(self) -> List[BDevice]:
+	def devices(self) -> list[BDevice]:
 		return list(self._devices.values())
 
 	def load_devices(self) -> None:
 		block_devices = {}
 
-		SysCommand('udevadm settle')
+		self.udev_sync()
 		all_lsblk_info = get_all_lsblk_info()
 		devices = getAllDevices()
+		devices.extend(self.get_loop_devices())
 
-		try:
-			loop_devices = SysCommand(['losetup', '-a'])
-			for ld_info in str(loop_devices).splitlines():
-				loop_device = getDevice(ld_info.split(':', maxsplit=1)[0])
-				devices.append(loop_device)
-		except Exception as err:
-			debug(f'Failed to get loop devices: {err}')
+		archiso_mountpoint = Path('/run/archiso/airootfs')
 
 		for device in devices:
 			dev_lsblk_info = find_lsblk_info(device.path, all_lsblk_info)
@@ -69,6 +77,10 @@ class DeviceHandler(object):
 			if dev_lsblk_info.type == 'rom':
 				continue
 
+			# exclude archiso loop device
+			if dev_lsblk_info.mountpoint == archiso_mountpoint:
+				continue
+
 			try:
 				if dev_lsblk_info.pttype:
 					disk = newDisk(device)
@@ -111,11 +123,35 @@ class DeviceHandler(object):
 
 		self._devices = block_devices
 
+	@staticmethod
+	def get_loop_devices() -> list[Device]:
+		devices = []
+
+		try:
+			loop_devices = SysCommand(['losetup', '-a'])
+		except SysCallError as err:
+			debug(f'Failed to get loop devices: {err}')
+		else:
+			for ld_info in str(loop_devices).splitlines():
+				try:
+					loop_device_path, _ = ld_info.split(':', maxsplit=1)
+				except ValueError:
+					continue
+
+				try:
+					loop_device = getDevice(loop_device_path)
+				except IOException as err:
+					debug(f'Failed to get loop device: {err}')
+				else:
+					devices.append(loop_device)
+
+		return devices
+
 	def _determine_fs_type(
 		self,
 		partition: Partition,
-		lsblk_info: Optional[LsblkInfo] = None
-	) -> Optional[FilesystemType]:
+		lsblk_info: LsblkInfo | None = None
+	) -> FilesystemType | None:
 		try:
 			if partition.fileSystem:
 				return FilesystemType(partition.fileSystem.type)
@@ -127,17 +163,17 @@ class DeviceHandler(object):
 
 		return None
 
-	def get_device(self, path: Path) -> Optional[BDevice]:
+	def get_device(self, path: Path) -> BDevice | None:
 		return self._devices.get(path, None)
 
-	def get_device_by_partition_path(self, partition_path: Path) -> Optional[BDevice]:
+	def get_device_by_partition_path(self, partition_path: Path) -> BDevice | None:
 		partition = self.find_partition(partition_path)
 		if partition:
 			device: Device = partition.disk.device
 			return self.get_device(Path(device.path))
 		return None
 
-	def find_partition(self, path: Path) -> Optional[_PartitionInfo]:
+	def find_partition(self, path: Path) -> _PartitionInfo | None:
 		for device in self._devices.values():
 			part = next(filter(lambda x: str(x.path) == str(path), device.partition_infos), None)
 			if part is not None:
@@ -148,11 +184,13 @@ class DeviceHandler(object):
 		lsblk = get_lsblk_info(dev_path)
 		return Path(f'/dev/{lsblk.pkname}')
 
-	def get_unique_path_for_device(self, dev_path: Path) -> Optional[Path]:
+	def get_unique_path_for_device(self, dev_path: Path) -> Path | None:
 		paths = Path('/dev/disk/by-id').glob('*')
 		linked_targets = {p.resolve(): p for p in paths}
-		linked_wwn_targets = {p: linked_targets[p] for p in linked_targets
-			if p.name.startswith('wwn-') or p.name.startswith('nvme-eui.')}
+		linked_wwn_targets = {
+			p: linked_targets[p] for p in linked_targets
+			if p.name.startswith('wwn-') or p.name.startswith('nvme-eui.')
+		}
 
 		if dev_path in linked_wwn_targets:
 			return linked_wwn_targets[dev_path]
@@ -162,19 +200,19 @@ class DeviceHandler(object):
 
 		return None
 
-	def get_uuid_for_path(self, path: Path) -> Optional[str]:
+	def get_uuid_for_path(self, path: Path) -> str | None:
 		partition = self.find_partition(path)
 		return partition.partuuid if partition else None
 
 	def get_btrfs_info(
 		self,
 		dev_path: Path,
-		lsblk_info: Optional[LsblkInfo] = None
-	) -> List[_BtrfsSubvolumeInfo]:
+		lsblk_info: LsblkInfo | None = None
+	) -> list[_BtrfsSubvolumeInfo]:
 		if not lsblk_info:
 			lsblk_info = get_lsblk_info(dev_path)
 
-		subvol_infos: List[_BtrfsSubvolumeInfo] = []
+		subvol_infos: list[_BtrfsSubvolumeInfo] = []
 
 		if not lsblk_info.mountpoint:
 			self.mount(dev_path, self._TMP_BTRFS_MOUNT, create_target_mountpoint=True)
@@ -194,13 +232,20 @@ class DeviceHandler(object):
 			debug(f'Failed to read btrfs subvolume information: {err}')
 			return subvol_infos
 
+		# It is assumed that lsblk will contain the fields as
+		# "mountpoints": ["/mnt/archinstall/log", "/mnt/archinstall/home", "/mnt/archinstall", ...]
+		# "fsroots": ["/@log", "/@home", "/@"...]
+		# we'll thereby map the fsroot, which are the mounted filesystem roots
+		# to the corresponding mountpoints
+		btrfs_subvol_info = dict(zip(lsblk_info.fsroots, lsblk_info.mountpoints))
+
 		try:
 			# ID 256 gen 16 top level 5 path @
 			for line in result.splitlines():
 				# expected output format:
 				# ID 257 gen 8 top level 5 path @home
 				name = Path(line.split(' ')[-1])
-				sub_vol_mountpoint = lsblk_info.btrfs_subvol_info.get(name, None)
+				sub_vol_mountpoint = btrfs_subvol_info.get(name, None)
 				subvol_infos.append(_BtrfsSubvolumeInfo(name, sub_vol_mountpoint))
 		except json.decoder.JSONDecodeError as err:
 			error(f"Could not decode lsblk JSON: {result}")
@@ -215,7 +260,7 @@ class DeviceHandler(object):
 		self,
 		fs_type: FilesystemType,
 		path: Path,
-		additional_parted_options: List[str] = []
+		additional_parted_options: list[str] = []
 	) -> None:
 		mkfs_type = fs_type.value
 		options = []
@@ -253,7 +298,7 @@ class DeviceHandler(object):
 	def encrypt(
 		self,
 		dev_path: Path,
-		mapper_name: Optional[str],
+		mapper_name: str | None,
 		enc_password: str,
 		lock_after_create: bool = True
 	) -> Luks2:
@@ -279,7 +324,7 @@ class DeviceHandler(object):
 	def format_encrypted(
 		self,
 		dev_path: Path,
-		mapper_name: Optional[str],
+		mapper_name: str | None,
 		fs_type: FilesystemType,
 		enc_conf: DiskEncryption
 	) -> None:
@@ -306,7 +351,7 @@ class DeviceHandler(object):
 		self,
 		cmd: str,
 		info_type: Literal['lv', 'vg', 'pvseg']
-	) -> Optional[Any]:
+	) -> Any | None:
 		raw_info = SysCommand(cmd).decode().split('\n')
 
 		# for whatever reason the output sometimes contains
@@ -344,18 +389,14 @@ class DeviceHandler(object):
 
 		return None
 
-	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['lv', 'vg', 'pvseg']) -> Optional[Any]:
-		attempts = 3
-
-		for attempt_nr in range(attempts):
+	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['lv', 'vg', 'pvseg']) -> Any | None:
+		while True:
 			try:
 				return self._lvm_info(cmd, info_type)
 			except ValueError:
-				time.sleep(attempt_nr + 1)
+				time.sleep(3)
 
-		raise ValueError(f'Failed to fetch {info_type} information')
-
-	def lvm_vol_info(self, lv_name: str) -> Optional[LvmVolumeInfo]:
+	def lvm_vol_info(self, lv_name: str) -> LvmVolumeInfo | None:
 		cmd = (
 			'lvs --reportformat json '
 			'--unit B '
@@ -364,7 +405,7 @@ class DeviceHandler(object):
 
 		return self._lvm_info_with_retry(cmd, 'lv')
 
-	def lvm_group_info(self, vg_name: str) -> Optional[LvmGroupInfo]:
+	def lvm_group_info(self, vg_name: str) -> LvmGroupInfo | None:
 		cmd = (
 			'vgs --reportformat json '
 			'--unit B '
@@ -374,7 +415,7 @@ class DeviceHandler(object):
 
 		return self._lvm_info_with_retry(cmd, 'vg')
 
-	def lvm_pvseg_info(self, vg_name: str, lv_name: str) -> Optional[LvmPVInfo]:
+	def lvm_pvseg_info(self, vg_name: str, lv_name: str) -> LvmPVInfo | None:
 		cmd = (
 			'pvs '
 			'--segments -o+lv_name,vg_name '
@@ -428,7 +469,7 @@ class DeviceHandler(object):
 		worker.poll()
 		worker.write(b'y\n', line_ending=False)
 
-	def lvm_vol_create(self, vg_name: str, volume: LvmVolume, offset: Optional[Size] = None) -> None:
+	def lvm_vol_create(self, vg_name: str, volume: LvmVolume, offset: Size | None = None) -> None:
 		if offset is not None:
 			length = volume.length - offset
 		else:
@@ -493,7 +534,7 @@ class DeviceHandler(object):
 		)
 
 		for flag in part_mod.flags:
-			partition.setFlag(flag.value)
+			partition.setFlag(flag.flag_id)
 
 		debug(f'\tType: {part_mod.type.value}')
 		debug(f'\tFilesystem: {part_mod.safe_fs_type.value}')
@@ -504,9 +545,11 @@ class DeviceHandler(object):
 		except PartitionException as ex:
 			raise DiskError(f'Unable to add partition, most likely due to overlapping sectors: {ex}') from ex
 
-		if disk.type == PartitionTable.GPT.value and part_mod.is_root():
-			linux_root_x86_64 = "4F68BCE3-E8CD-4DB1-96E7-FBCAF984B709"
-			partition.type_uuid = uuid.UUID(linux_root_x86_64).bytes
+		if disk.type == PartitionTable.GPT.value:
+			if part_mod.is_root():
+				partition.type_uuid = PartitionGUID.LINUX_ROOT_X86_64.bytes
+			elif PartitionFlag.LINUX_HOME not in part_mod.flags and part_mod.is_home():
+				partition.setFlag(PartitionFlag.LINUX_HOME.flag_id)
 
 		# the partition has a path now that it has been added
 		part_mod.dev_path = Path(partition.path)
@@ -533,8 +576,8 @@ class DeviceHandler(object):
 	def create_lvm_btrfs_subvolumes(
 		self,
 		path: Path,
-		btrfs_subvols: List[SubvolumeModification],
-		mount_options: List[str]
+		btrfs_subvols: list[SubvolumeModification],
+		mount_options: list[str]
 	) -> None:
 		info(f'Creating subvolumes: {path}')
 
@@ -564,7 +607,7 @@ class DeviceHandler(object):
 	def create_btrfs_volumes(
 		self,
 		part_mod: PartitionModification,
-		enc_conf: Optional['DiskEncryption'] = None
+		enc_conf: 'DiskEncryption | None' = None
 	) -> None:
 		info(f'Creating subvolumes: {part_mod.safe_dev_path}')
 
@@ -634,7 +677,7 @@ class DeviceHandler(object):
 	def partition(
 		self,
 		modification: DeviceModification,
-		partition_table: Optional[PartitionTable] = None
+		partition_table: PartitionTable | None = None
 	) -> None:
 		"""
 		Create a partition table on the block device and create all partitions.
@@ -672,9 +715,9 @@ class DeviceHandler(object):
 		self,
 		dev_path: Path,
 		target_mountpoint: Path,
-		mount_fs: Optional[str] = None,
+		mount_fs: str | None = None,
 		create_target_mountpoint: bool = True,
-		options: List[str] = []
+		options: list[str] = []
 	) -> None:
 		if create_target_mountpoint and not target_mountpoint.exists():
 			target_mountpoint.mkdir(parents=True, exist_ok=True)
@@ -722,8 +765,8 @@ class DeviceHandler(object):
 			debug(f'Unmounting mountpoint: {path}')
 			SysCommand(cmd + [str(path)])
 
-	def detect_pre_mounted_mods(self, base_mountpoint: Path) -> List[DeviceModification]:
-		part_mods: Dict[Path, List[PartitionModification]] = {}
+	def detect_pre_mounted_mods(self, base_mountpoint: Path) -> list[DeviceModification]:
+		part_mods: dict[Path, list[PartitionModification]] = {}
 
 		for device in self.devices:
 			for part_info in device.partition_infos:
@@ -741,14 +784,14 @@ class DeviceHandler(object):
 						part_mods[path].append(part_mod)
 						break
 
-		device_mods: List[DeviceModification] = []
+		device_mods: list[DeviceModification] = []
 		for device_path, mods in part_mods.items():
 			device_mod = DeviceModification(self._devices[device_path], False, mods)
 			device_mods.append(device_mod)
 
 		return device_mods
 
-	def partprobe(self, path: Optional[Path] = None) -> None:
+	def partprobe(self, path: Path | None = None) -> None:
 		if path is not None:
 			command = f'partprobe {path}'
 		else:
@@ -780,21 +823,30 @@ class DeviceHandler(object):
 		"""
 		info(f'Wiping partitions and metadata: {block_device.device_info.path}')
 		for partition in block_device.partition_infos:
+			luks = Luks2(partition.path)
+			if luks.isLuks():
+				luks.erase()
+
 			self._wipe(partition.path)
 
 		self._wipe(block_device.device_info.path)
 
+	@staticmethod
+	def udev_sync() -> None:
+		try:
+			SysCommand('udevadm settle')
+		except SysCallError as err:
+			debug(f'Failed to synchronize with udev: {err}')
+
 
 device_handler = DeviceHandler()
 
 
 def disk_layouts() -> str:
 	try:
-		lsblk_info = get_all_lsblk_info()
-		return json.dumps(lsblk_info, indent=4, sort_keys=True, cls=JSON)
+		lsblk_output = get_lsblk_output()
 	except SysCallError as err:
 		warn(f"Could not return disk layouts: {err}")
 		return ''
-	except json.decoder.JSONDecodeError as err:
-		warn(f"Could not return disk layouts: {err}")
-		return ''
+
+	return lsblk_output.model_dump_json(indent=4)
