@@ -2,9 +2,9 @@ from .locale_menu import LocaleConfiguration
 from .utils import (
 	list_keyboard_languages,
 	list_locales,
+	list_timezones,
 	list_x11_keyboard_languages,
+	set_kb_layout,
 	verify_keyboard_layout,
 	verify_x11_keyboard_layout,
-	list_timezones,
-	set_kb_layout
 )
