@@ -1,22 +1,23 @@
 import os
+import random
+import select
+import signal
 import socket
 import ssl
 import struct
 import time
-import select
-import signal
-import random
-from typing import Union, Dict, Any, List, Optional
+from types import FrameType
+from typing import Any
 from urllib.error import URLError
 from urllib.parse import urlencode
 from urllib.request import urlopen
 
-from .exceptions import SysCallError, DownloadTimeout
-from .output import error, info
+from .exceptions import DownloadTimeout, SysCallError
+from .output import debug, error, info
 from .pacman import Pacman
 
 
-class DownloadTimer():
+class DownloadTimer:
 	'''
 	Context manager for timing downloads with timeouts.
 	'''
@@ -27,13 +28,13 @@ class DownloadTimer():
 				The download timeout in seconds. The DownloadTimeout exception
 				will be raised in the context after this many seconds.
 		'''
-		self.time: Optional[float] = None
-		self.start_time: Optional[float] = None
+		self.time: float | None = None
+		self.start_time: float | None = None
 		self.timeout = timeout
 		self.previous_handler = None
-		self.previous_timer: Optional[int] = None
+		self.previous_timer: int | None = None
 
-	def raise_timeout(self, signl, frame) -> None:
+	def raise_timeout(self, signl: int, frame: FrameType | None) -> None:
 		'''
 		Raise the DownloadTimeout exception.
 		'''
@@ -73,7 +74,7 @@ def get_hw_addr(ifname: str) -> str:
 	return ':'.join('%02x' % b for b in ret[18:24])
 
 
-def list_interfaces(skip_loopback: bool = True) -> Dict[str, str]:
+def list_interfaces(skip_loopback: bool = True) -> dict[str, str]:
 	interfaces = {}
 
 	for index, iface in socket.if_nameindex():
@@ -98,7 +99,7 @@ def update_keyring() -> bool:
 	return False
 
 
-def enrich_iface_types(interfaces: Union[Dict[str, Any], List[str]]) -> Dict[str, str]:
+def enrich_iface_types(interfaces: dict[str, Any] | list[str]) -> dict[str, str]:
 	result = {}
 
 	for iface in interfaces:
@@ -118,7 +119,7 @@ def enrich_iface_types(interfaces: Union[Dict[str, Any], List[str]]) -> Dict[str
 	return result
 
 
-def fetch_data_from_url(url: str, params: Optional[Dict] = None) -> str:
+def fetch_data_from_url(url: str, params: dict[str, str] | None = None) -> str:
 	ssl_context = ssl.create_default_context()
 	ssl_context.check_hostname = False
 	ssl_context.verify_mode = ssl.CERT_NONE
@@ -133,11 +134,13 @@ def fetch_data_from_url(url: str, params: Optional[Dict] = None) -> str:
 		response = urlopen(full_url, context=ssl_context)
 		data = response.read().decode('UTF-8')
 		return data
-	except URLError:
-		raise ValueError(f'Unable to fetch data from url: {url}')
+	except URLError as e:
+		raise ValueError(f'Unable to fetch data from url: {url}\n{e}')
+	except Exception as e:
+		raise ValueError(f'Unexpected error when parsing response: {e}')
 
 
-def calc_checksum(icmp_packet) -> int:
+def calc_checksum(icmp_packet: bytes) -> int:
 	# Calculate the ICMP checksum
 	checksum = 0
 	for i in range(0, len(icmp_packet), 2):
@@ -188,8 +191,8 @@ def ping(hostname, timeout=5) -> int:
 				if icmp_type == 0 and response[-len(random_identifier):] == random_identifier:
 					latency = round((time.time() - started) * 1000)
 					break
-		except socket.error as error:
-			print(f"Error: {error}")
+		except OSError as error:
+			debug(f"Error: {error}")
 			break
 
 	icmp_socket.close()
