@@ -2,11 +2,11 @@ import logging
 import os
 import sys
 import unicodedata
-from enum import Enum
-
-from pathlib import Path
-from typing import Dict, Union, List, Any, Callable, Optional, TYPE_CHECKING
+from collections.abc import Callable
 from dataclasses import asdict, is_dataclass
+from enum import Enum
+from pathlib import Path
+from typing import TYPE_CHECKING, Any
 
 from .storage import storage
 
@@ -20,9 +20,9 @@ class FormattedOutput:
 	def _get_values(
 		cls,
 		o: 'DataclassInstance',
-		class_formatter: Optional[Union[str, Callable]] = None,
-		filter_list: List[str] = []
-	) -> Dict[str, Any]:
+		class_formatter: str | Callable | None = None,
+		filter_list: list[str] = []
+	) -> dict[str, Any]:
 		"""
 		the original values returned a dataclass as dict thru the call to some specific methods
 		this version allows thru the parameter class_formatter to call a dynamically selected formatting method.
@@ -51,9 +51,9 @@ class FormattedOutput:
 	@classmethod
 	def as_table(
 		cls,
-		obj: List[Any],
-		class_formatter: Optional[Union[str, Callable]] = None,
-		filter_list: List[str] = [],
+		obj: list[Any],
+		class_formatter: str | Callable | None = None,
+		filter_list: list[str] = [],
 		capitalize: bool = False
 	) -> str:
 		""" variant of as_table (subtly different code) which has two additional parameters
@@ -67,7 +67,7 @@ class FormattedOutput:
 		raw_data = [cls._get_values(o, class_formatter, filter_list) for o in obj]
 
 		# determine the maximum column size
-		column_width: Dict[str, int] = {}
+		column_width: dict[str, int] = {}
 		for o in raw_data:
 			for k, v in o.items():
 				if not filter_list or k in filter_list:
@@ -100,9 +100,9 @@ class FormattedOutput:
 				value = record.get(key, '')
 
 				if '!' in key:
-					value = '*' * width
+					value = '*' * len(value)
 
-				if isinstance(value, (int, float)) or (isinstance(value, str) and value.isnumeric()):
+				if isinstance(value, int | float) or (isinstance(value, str) and value.isnumeric()):
 					obj_data.append(unicode_rjust(str(value), width))
 				else:
 					obj_data.append(unicode_ljust(str(value), width))
@@ -112,7 +112,7 @@ class FormattedOutput:
 		return output
 
 	@classmethod
-	def as_columns(cls, entries: List[str], cols: int) -> str:
+	def as_columns(cls, entries: list[str], cols: int) -> str:
 		"""
 		Will format a list into a given number of columns
 		"""
@@ -202,9 +202,9 @@ class Font(Enum):
 def _stylize_output(
 	text: str,
 	fg: str,
-	bg: Optional[str],
+	bg: str | None,
 	reset: bool,
-	font: List[Font] = [],
+	font: list[Font] = [],
 ) -> str:
 	"""
 	Heavily influenced by:
@@ -256,9 +256,9 @@ def info(
 	*msgs: str,
 	level: int = logging.INFO,
 	fg: str = 'white',
-	bg: Optional[str] = None,
+	bg: str | None = None,
 	reset: bool = False,
-	font: List[Font] = []
+	font: list[Font] = []
 ) -> None:
 	log(*msgs, level=level, fg=fg, bg=bg, reset=reset, font=font)
 
@@ -267,9 +267,9 @@ def debug(
 	*msgs: str,
 	level: int = logging.DEBUG,
 	fg: str = 'white',
-	bg: Optional[str] = None,
+	bg: str | None = None,
 	reset: bool = False,
-	font: List[Font] = []
+	font: list[Font] = []
 ) -> None:
 	log(*msgs, level=level, fg=fg, bg=bg, reset=reset, font=font)
 
@@ -278,20 +278,20 @@ def error(
 	*msgs: str,
 	level: int = logging.ERROR,
 	fg: str = 'red',
-	bg: Optional[str] = None,
+	bg: str | None = None,
 	reset: bool = False,
-	font: List[Font] = []
+	font: list[Font] = []
 ) -> None:
 	log(*msgs, level=level, fg=fg, bg=bg, reset=reset, font=font)
 
 
 def warn(
 	*msgs: str,
-	level: int = logging.WARN,
+	level: int = logging.WARNING,
 	fg: str = 'yellow',
-	bg: Optional[str] = None,
+	bg: str | None = None,
 	reset: bool = False,
-	font: List[Font] = []
+	font: list[Font] = []
 ) -> None:
 	log(*msgs, level=level, fg=fg, bg=bg, reset=reset, font=font)
 
@@ -300,9 +300,9 @@ def log(
 	*msgs: str,
 	level: int = logging.INFO,
 	fg: str = 'white',
-	bg: Optional[str] = None,
+	bg: str | None = None,
 	reset: bool = False,
-	font: List[Font] = []
+	font: list[Font] = []
 ) -> None:
 	# leave this check here as we need to setup the logging
 	# right from the beginning when the modules are loaded
@@ -322,14 +322,9 @@ def log(
 
 	Journald.log(text, level=level)
 
-	from .menu import Menu
-	if not Menu.is_menu_active():
-		# Finally, print the log unless we skipped it based on level.
-		# We use sys.stdout.write()+flush() instead of print() to try and
-		# fix issue #94
-		if level != logging.DEBUG or storage.get('arguments', {}).get('verbose', False):
-			sys.stdout.write(f"{text}\n")
-			sys.stdout.flush()
+	if level != logging.DEBUG or storage.get('arguments', {}).get('verbose', False):
+		from archinstall.tui import Tui
+		Tui.print(text)
 
 
 def _count_wchars(string: str) -> int:
