@@ -1,7 +1,6 @@
 import re
 from pathlib import Path
 from shutil import copy2
-from typing import List
 
 from .repo import Repo
 
@@ -10,7 +9,7 @@ class Config:
 	def __init__(self, target: Path):
 		self.path = Path("/etc") / "pacman.conf"
 		self.chroot_path = target / "etc" / "pacman.conf"
-		self.repos: List[Repo] = []
+		self.repos: list[Repo] = []
 
 	def enable(self, repo: Repo) -> None:
 		self.repos.append(repo)
